package cz.jzitnik.chronos.controllers;

import cz.jzitnik.chronos.entities.Room;
import cz.jzitnik.chronos.payload.responses.UnifiedResponse;
import cz.jzitnik.chronos.repository.PlayerRepository;
import cz.jzitnik.chronos.repository.RoomRepository;
import cz.jzitnik.chronos.services.GameService;
import cz.jzitnik.chronos.utils.anotations.CheckUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import java.util.List;

@CrossOrigin(origins = "*", maxAge = 3600)
@RestController
@RequestMapping("/game/rooms")
public class RoomController {
    @Autowired
    private PlayerRepository playerRepository;

    @Autowired
    private RoomRepository roomRepository;

    @Autowired
    private GameService gameService;

    @GetMapping
    @CheckUser
    public ResponseEntity<UnifiedResponse<List<Room>, Error>> getAllRooms(@RequestParam("playerKey") String playerKey) {
        var player = playerRepository.findByPlayerKey(playerKey).get();

        return ResponseEntity.ok(
                UnifiedResponse.success(player.getGame().getRooms())
        );
    }

    @GetMapping("/current_room")
    @CheckUser
    public ResponseEntity<UnifiedResponse<Room, Error>> getCurrentRoom(@RequestParam("playerKey") String playerKey) {
        var player = playerRepository.findByPlayerKey(playerKey).get();

        return ResponseEntity.ok(
                UnifiedResponse.success(player.getCurrentRoom())
        );
    }

    @PostMapping("/move")
    @CheckUser
    public ResponseEntity<UnifiedResponse<Object, Error>> moveToRoom(@RequestParam("playerKey") String playerKey, @RequestParam("roomId") Long roomId) {
        var player = playerRepository.findByPlayerKey(playerKey).get();

        var roomOptional = roomRepository.findById(roomId);

        if (roomOptional.isEmpty()) {
            return ResponseEntity.status(HttpStatus.NOT_FOUND).body(UnifiedResponse.failure(null));
        }

        var fromThisGame = gameService.isFromThisGame(player.getGame(), roomOptional.get());

        if (!fromThisGame) {
            return ResponseEntity.status(HttpStatus.BAD_REQUEST).body(UnifiedResponse.failure(new Error("Invalid roomId")));
        }

        player.setCurrentRoom(roomOptional.get());

        playerRepository.save(player);

        return ResponseEntity.ok(UnifiedResponse.success(null));
    }
}
